@@ -142,6 +142,15 @@ class Mailbox:
             return
 
         # nope. delete and summarize
+
+        # if the nameplate is still allocated we'll get a foreign-key
+        # failure when trying to delete the mailbox, so get rid of
+        # those first
+        db.execute("DELETE FROM `nameplate_sides` WHERE `side`=?",
+                   (side,))
+        db.execute("DELETE FROM `nameplates` WHERE `mailbox_id`=?",
+                   (self._mailbox_id,))
+        # remove mailbox content
         db.execute("DELETE FROM `messages` WHERE `mailbox_id`=?",
                    (self._mailbox_id,))
         db.execute("DELETE FROM `mailbox_sides` WHERE `mailbox_id`=?",
@@ -183,13 +192,14 @@ class AppNamespace(object):
             server_rx = self._blur_usage * (server_rx // self._blur_usage)
         implementation = client_version[0]
         version = client_version[1]
-        self._usage_db.execute("INSERT INTO `client_versions`"
-                               " (`app_id`, `side`, `connect_time`,"
-                               "  `implementation`, `version`)"
-                               " VALUES(?,?,?,?,?)",
-                               (self._app_id, side, server_rx,
-                                implementation, version))
-        self._usage_db.commit()
+        if self._usage_db:
+            self._usage_db.execute("INSERT INTO `client_versions`"
+                                   " (`app_id`, `side`, `connect_time`,"
+                                   "  `implementation`, `version`)"
+                                   " VALUES(?,?,?,?,?)",
+                                   (self._app_id, side, server_rx,
+                                    implementation, version))
+            self._usage_db.commit()
 
     def get_nameplate_ids(self):
         if not self._allow_list:
@@ -521,14 +531,16 @@ class AppNamespace(object):
                        (mailbox_id,))
             if self._usage_db:
                 self._summarize_mailbox_and_store(for_nameplate, side_rows,
-                                                now, pruned=True)
+                                                  now, pruned=True)
             modified = True
 
         if modified:
             db.commit()
             if self._usage_db:
                 self._usage_db.commit()
-        log.msg("  prune complete, modified:", modified)
+        in_use = bool(self._mailboxes)
+        log.msg("  prune complete, modified=%s, in_use=%s" % (modified, in_use))
+        return in_use
 
     def count_listeners(self):
         return sum(mailbox.count_listeners()
@@ -591,7 +603,9 @@ class Server(service.MultiService):
         for app_id in sorted(self.get_all_apps()):
             log.msg(" app prune checking %r" % (app_id,))
             app = self.get_app(app_id)
-            app.prune(now, old)
+            in_use = app.prune(now, old)
+            if not in_use:
+                del self._apps[app_id]
         log.msg("app prune ends, %d apps" % len(self._apps))
 
     def dump_stats(self, now, rebooted):
@@ -663,24 +677,23 @@ class Server(service.MultiService):
         return service.MultiService.stopService(self)
 
 def make_server(db, allow_list=True,
-                advertise_version=None, signal_error=None,
+                advertise_version=None,
+                signal_error=None,
                 blur_usage=None,
                 usage_db=None,
                 log_file=None,
+                welcome_motd=None,
                 ):
     if blur_usage:
         log.msg("blurring access times to %d seconds" % blur_usage)
     else:
         log.msg("not blurring access times")
 
-    welcome = {
+    welcome = dict()
+    if welcome_motd is not None:
         # adding .motd will cause all clients to display the message,
         # then keep running normally
-        #"motd": "Welcome to the public relay.\nPlease enjoy this service.",
-
-        # adding .error will cause all clients to fail, with this message
-        #"error": "This server has been disabled, see URL for details.",
-        }
+        welcome["motd"] = str(welcome_motd)
 
     if advertise_version:
         # The primary (python CLI) implementation will emit a message if
@@ -688,7 +701,9 @@ def make_server(db, allow_list=True,
         # distributions which include older version, but we still expect
         # them to be compatible, stop sending this key.
         welcome["current_cli_version"] = advertise_version
+
     if signal_error:
         welcome["error"] = signal_error
+
     return Server(db, allow_list=allow_list, welcome=welcome,
                   blur_usage=blur_usage, usage_db=usage_db, log_file=log_file)
